@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2007 Kurt Hornik
+## Copyright (C) 1995, 1996, 1997, 2005, 2007, 2008 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -26,8 +26,17 @@
 ## Author: TT <Teresa.Twaroch@ci.tuwien.ac.at>
 ## Description: Quantile function of the chi-square distribution
 
+## Deprecated in version 3.0
+
 function inv = chisquare_inv (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "chisquare_inv is obsolete and will be removed from a future version of Octave; please use chi2inv instead");
+  endif
+
  inv =  chi2inv (varargin{:});
 
 endfunction
