@@ -1,4 +1,4 @@
-## Copyright (C) 2003, 2005, 2007 John W. Eaton
+## Copyright (C) 2003, 2005, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -23,8 +23,17 @@
 
 ## Author: jwe
 
+## Deprecated in version 3.0
+
 function retval = struct_contains (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "struct_contains is obsolete and will be removed from a future version of Octave; please use isfield instead");
+  endif
+
   retval = isfield (varargin{:});
 
 endfunction
