@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2005, 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -30,8 +30,17 @@
 ## for @var{x} >= 0.
 ## @end deftypefn
 
+## Deprecated in version 3.0
+
 function cdf = weibull_cdf (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "weibull_cdf is obsolete and will be removed from a future version of Octave; please use wblcdf instead");
+  endif
+
   if (nargin == 2)
     varargin{3} = varargin{2};
     varargin{2} = 1;
