@@ -1,4 +1,4 @@
-## Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+## Copyright (C) 2004, 2005, 2006, 2007, 2008 David Bateman
 ##
 ## This file is part of Octave.
 ##
@@ -48,37 +48,49 @@
 function y = circshift (x, n)
 
   if (nargin == 2)
-    nd = ndims (x);
-    sz = size (x);
+    if (isempty (x))
+      y = x;
+    else
+      nd = ndims (x);
+      sz = size (x);
 
-    if (! isvector (n) && length (n) > nd)
-      error ("circshift: n must be a vector, no longer than the number of dimension in x");
-    endif
-    
-    if (any (n != floor (n)))
-      error ("circshift: all values of n must be integers");
-    endif
-
-    idx = cell ();
-    for i = 1:length (n);
-      nn = n(i);
-      if (nn < 0)
-	while (sz(i) <= -nn)
-	  nn = nn + sz(i);
-	endwhile
-	idx{i} = [(1-nn):sz(i), 1:-nn];
-      else
-	while (sz(i) <= nn)
-	  nn = nn - sz(i);
-	endwhile
-	idx{i} = [(sz(i)-nn+1):sz(i), 1:(sz(i)-nn)];
+      if (! isvector (n) && length (n) > nd)
+	error ("circshift: n must be a vector, no longer than the number of dimension in x");
       endif
-    endfor
-    for i = (length(n) + 1) : nd
-      idx{i} = 1:sz(i);
-    endfor
-    y = x(idx{:});
+    
+      if (any (n != floor (n)))
+	error ("circshift: all values of n must be integers");
+      endif
+
+      idx = cell ();
+      for i = 1:length (n);
+	nn = n(i);
+	if (nn < 0)
+	  while (sz(i) <= -nn)
+	    nn = nn + sz(i);
+	  endwhile
+	  idx{i} = [(1-nn):sz(i), 1:-nn];
+	else
+	  while (sz(i) <= nn)
+	    nn = nn - sz(i);
+	  endwhile
+	  idx{i} = [(sz(i)-nn+1):sz(i), 1:(sz(i)-nn)];
+	endif
+      endfor
+      for i = (length(n) + 1) : nd
+	idx{i} = 1:sz(i);
+      endfor
+      y = x(idx{:});
+    endif
   else
     print_usage ();
   endif
 endfunction
+
+%!shared x
+%! x = [1, 2, 3; 4, 5, 6; 7, 8, 9];
+
+%!assert (circshift (x, 1), [7, 8, 9; 1, 2, 3; 4, 5, 6])
+%!assert (circshift (x, -2), [7, 8, 9; 1, 2, 3; 4, 5, 6])
+%!assert (circshift (x, [0, 1]), [3, 1, 2; 6, 4, 5; 9, 7, 8]);
+%!assert (circshift ([],1), [])
