@@ -1,5 +1,5 @@
 ## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003,
-##               2005, 2006, 2007 John W. Eaton
+##               2005, 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -44,7 +44,7 @@ function retval = ylabel (varargin)
   end_unwind_protect
 
   if (nargout > 0)
-    retval = h;
+    retval = tmp;
   endif
 
 endfunction
