@@ -24,7 +24,7 @@
 ##
 ## @example
 ## @group
-## asctime (localtime (time ())
+## asctime (localtime (time ()))
 ##      @result{} "Mon Feb 17 01:15:06 1997\n"
 ## @end group
 ## @end example
