@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2006, 2007 John W. Eaton
+Copyright (C) 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -346,13 +346,18 @@ maybe_add_path_elts (std::string& path, const std::string& dir)
   std::string tpath = genpath (dir);
 
   if (! tpath.empty ())
-    path += dir_path::path_sep_str + tpath;
+    {
+      if (path.empty ())
+	path = tpath;
+      else
+	path += dir_path::path_sep_str + tpath;
+    }
 }
 
 void
 load_path::do_initialize (bool set_initial_path)
 {
-  sys_path = dir_path::path_sep_str;
+  sys_path = "";
 
   if (set_initial_path)
     {
@@ -369,15 +374,15 @@ load_path::do_initialize (bool set_initial_path)
   std::string tpath = load_path::command_line_path;
 
   if (tpath.empty ())
-    tpath = octave_env::getenv ("OCTAVE_LOADPATH");
+    tpath = octave_env::getenv ("OCTAVE_PATH");
 
   std::string xpath = ".";
 
   if (! tpath.empty ())
     xpath += dir_path::path_sep_str + tpath;
 
-  if (sys_path != dir_path::path_sep_str)
-    xpath += sys_path;
+  if (! sys_path.empty ())
+    xpath += dir_path::path_sep_str + sys_path;
 
   do_set (xpath, false);
 }
@@ -788,6 +793,50 @@ load_path::do_find_file (const std::string& file) const
   return retval;
 }
 
+std::string
+load_path::do_find_dir (const std::string& dir) const
+{
+  std::string retval;
+
+  if (dir.find_first_of (file_ops::dir_sep_chars) != std::string::npos
+      && (octave_env::absolute_pathname (dir)
+	  || octave_env::rooted_relative_pathname (dir)))
+    {
+      file_stat fs (dir);
+
+      if (fs.exists () && fs.is_dir ())
+	return dir;
+    }
+  else
+    {
+      for (const_dir_info_list_iterator p = dir_info_list.begin ();
+	   p != dir_info_list.end ();
+	   p++)
+	{
+	  std::string dname = p->dir_name;
+
+	  size_t dname_len = dname.length ();
+
+	  if (dname.substr (dname_len - 1) == file_ops::dir_sep_str)
+	    dname = dname.substr (0, dname_len - 1);
+
+	  size_t dir_len = dir.length ();
+
+	  if (dname_len >= dir_len
+	      && file_ops::is_dir_sep (dname[dname_len - dir_len - 1])
+	      && dir.compare (dname.substr (dname_len - dir_len)) == 0)
+	    {
+	      file_stat fs (p->dir_name);
+
+	      if (fs.exists () && fs.is_dir ())
+		return p->dir_name;
+	    }
+	}
+    }
+
+  return retval;
+}
+
 std::string
 load_path::do_find_first_of (const string_vector& flist) const
 {
@@ -1249,7 +1298,7 @@ execute_pkg_del (const std::string& dir)
 DEFUN (genpath, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} genpath (@var{dir})\n\
-Return a path constructed from @var{dir} and all its subdiretories.\n\
+Return a path constructed from @var{dir} and all its subdirectories.\n\
 @end deftypefn")
 {
   octave_value retval;
