@@ -108,21 +108,15 @@ extract_keyword (std::istream& is, const char *keyword, const bool next_only)
 	      while (is.get (c) && (c == ' ' || c == '\t' || c == ':'))
 		; // Skip whitespace and the colon.
 
-	      if (c != '\n')
-		{
-		  value << c;
-		  while (is.get (c) && c != '\n')
-		    value << c;
-		}
-
-	      retval = value.str ();
+	      is.putback(c);
+	      retval = read_until_newline (is, false);
 	      break;
 	    }
 	  else if (next_only)
 	    break;
 	  else
 	    {
-	      while (is.get (c) && c != '\n')
+	      while (is.get (c) && c != '\n' && c != '\r')
 		; // Skip to end of line.
 	    }
 	}
