@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2005, 2006, 2007
+              John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -28,6 +28,8 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include "cmd-hist.h"
 
+extern void initialize_history (void);
+
 // Write timestamp to history file.
 extern void octave_history_write_timestamp (void);
 
