@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003,
+              2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -26,7 +26,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #endif
 
 #include <cfloat>
-#include <cmath>
 #include <cstddef>
 #include <cstdio>
 #include <cstdlib>
@@ -73,6 +72,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "cmd-edit.h"
 #include "file-ops.h"
 #include "lo-mappers.h"
+#include "lo-math.h"
 #include "mach-info.h"
 #include "oct-env.h"
 #include "quit.h"
@@ -109,17 +109,18 @@ BSD_init (void)
 }
 #endif
 
+#if defined (__WIN32__) && ! defined (_POSIX_VERSION)
 static void
 w32_set_octave_home (void)
 {
-#if defined (__WIN32__) && ! defined (_POSIX_VERSION)
   int n = 1024;
 
   std::string bin_dir (n, '\0');
 
   while (true)
     {
-      int status = GetModuleFileName (0, &bin_dir[0], n);
+      HMODULE hMod = GetModuleHandle ("octinterp");
+      int status = GetModuleFileName (hMod, &bin_dir[0], n);
 
       if (status < n)
 	{
@@ -140,22 +141,18 @@ w32_set_octave_home (void)
       if (pos != NPOS)
 	octave_env::putenv ("OCTAVE_HOME", bin_dir.substr (0, pos));
     }
-#endif
 }
 
 void
 w32_set_quiet_shutdown (void)
 {
-#if defined (__WIN32__) && ! defined (_POSIX_VERSION)
   // Let the user close the console window or shutdown without the
   // pesky dialog.
   //
   // FIXME -- should this be user configurable?
   SetProcessShutdownParameters (0x280, SHUTDOWN_NORETRY);
-#endif
 }
 
-#if defined (__WIN32__) && ! defined (_POSIX_VERSION)
 void
 MINGW_signal_cleanup (void)
 {
@@ -232,47 +229,38 @@ same_file_internal (const std::string& file1, const std::string& file2)
 {
 #ifdef OCTAVE_USE_WINDOWS_API
 
+  bool retval = false;
+
   // Windows native code 
   // Reference: http://msdn2.microsoft.com/en-us/library/aa363788.aspx
 
-  HANDLE hfile1;
-  HANDLE hfile2;
-  
-  BY_HANDLE_FILE_INFORMATION hfi1;
-  BY_HANDLE_FILE_INFORMATION hfi2;
-  
-  hfile1 = CreateFile (file1.c_str (), 0, FILE_SHARE_READ, 0,
-		       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0); 
+  HANDLE hfile1 = CreateFile (file1.c_str (), 0, FILE_SHARE_READ, 0,
+			      OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0); 
 
-  if (hfile1 == INVALID_FILE_HANDLE)
-    return false;
-  
-  hfile2 = CreateFile (file2.c_str (), 0, FILE_SHARE_READ, 0,
-		       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+  if (hfile1 != INVALID_HANDLE_VALUE)
+    {
+      HANDLE hfile2 = CreateFile (file2.c_str (), 0, FILE_SHARE_READ, 0,
+				  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
 
-  if (hfile2 == INVALID_FILE_HANDLE)
-    {
-      CloseHandle (hfile1);
-      return false;
-    }
+      if (hfile2 != INVALID_HANDLE_VALUE)
+	{  
+	  BY_HANDLE_FILE_INFORMATION hfi1;
+	  BY_HANDLE_FILE_INFORMATION hfi2;
   
-  if (! GetFileInformationByHandle (hfile1, &hfi1))
-    {
-      CloseHandle (hfile1);
-      CloseHandle (hfile2);
-      return false;
-    }
-   
-  if (! GetFileInformationByHandle (hfile2, &hfi2))
-    {
-      CloseHandle (hfile1);
-      CloseHandle (hfile2);
-      return false;
-    }
+	  if (GetFileInformationByHandle (hfile1, &hfi1)
+	      && GetFileInformationByHandle (hfile2, &hfi2))
   
-  return (hfi1.dwVolumeSerialNumber == hfi2.dwVolumeSerialNumber
-	  && hfi1.nFileIndexHigh == hfi2.nFileIndexHigh
-	  && hfi1.nFileIndexLow == hfi2.nFileIndexLow);
+	    retval = (hfi1.dwVolumeSerialNumber == hfi2.dwVolumeSerialNumber
+		      && hfi1.nFileIndexHigh == hfi2.nFileIndexHigh
+		      && hfi1.nFileIndexLow == hfi2.nFileIndexLow);
+
+	  CloseHandle (hfile2);
+	}
+
+      CloseHandle (hfile1);
+    }
+
+  return retval;
 
 #else
 
@@ -525,6 +513,36 @@ raw_mode (bool on, bool wait)
   curr_on = on;
 }
 
+FILE *
+octave_popen (const char *command, const char *mode)
+{
+#if defined (__MINGW32__) || defined (_MSC_VER)
+  if (mode && mode[0] && ! mode[1])
+    {
+      char tmode[3];
+      tmode[0] = mode[0];
+      tmode[1] = 'b';
+      tmode[2] = 0;
+
+      return _popen (command, tmode);
+    }
+  else
+    return _popen (command, mode);
+#else
+  return popen (command, mode);
+#endif
+}
+
+int
+octave_pclose (FILE *f)
+{
+#if defined (__MINGW32__) || defined (_MSC_VER)
+  return _pclose (f);
+#else
+  return pclose (f);
+#endif
+}
+
 // Read one character from the terminal.
 
 int
@@ -605,6 +623,7 @@ returns a string containing the value of your path.\n\
 DEFUN (putenv, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} putenv (@var{var}, @var{value})\n\
+@deftypefnx {Built-in Function} {} setenv (@var{var}, @var{value})\n\
 Set the value of the environment variable @var{var} to @var{value}.\n\
 @end deftypefn")
 {
@@ -612,13 +631,14 @@ Set the value of the environment variable @var{var} to @var{value}.\n\
 
   int nargin = args.length ();
 
-  if (nargin == 2)
+  if (nargin == 2 || nargin == 1)
     {
       std::string var = args(0).string_value (); 
 
       if (! error_state)
 	{
-	  std::string val = args(1).string_value (); 
+	  std::string val = (nargin == 2
+			     ? args(1).string_value () : std::string ()); 
 
 	  if (! error_state)
 	    octave_env::putenv (var, val);
@@ -633,6 +653,7 @@ Set the value of the environment variable @var{var} to @var{value}.\n\
 
   return retval;
 }
+DEFALIAS (setenv, putenv);
 
 // FIXME -- perhaps kbhit should also be able to print a prompt?
 
@@ -692,7 +713,7 @@ screen.\n\
 \n\
 @example\n\
 @group\n\
-fprintf (stderr, \"wait please...\n\");\n\
+fprintf (stderr, \"wait please...\\n\");\n\
 pause (5);\n\
 clc;\n\
 @end group\n\
